@@ -2,16 +2,20 @@
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "feature_select.h"
-#include "pysidecleanup.h"
-#include "pysideqobject.h"
+#include "basewrapper.h"
 #include "pysidestaticstrings.h"
 #include "class_property.h"
 
-#include <shiboken.h>
+#include <autodecref.h>
 #include <sbkfeature_base.h>
+#include <sbkpep.h>
+#include <sbkstaticstrings.h>
+#include <sbkstring.h>
 #include <signature_p.h>
 
-#include <QtCore/QStringList>
+#include <QtCore/qstringlist.h>
+
+#include <cstring>
 
 //////////////////////////////////////////////////////////////////////////////
 //
@@ -122,7 +126,7 @@ static void ensureNewDictType()
     if (new_dict_type == nullptr) {
         new_dict_type = createDerivedDictType();
         if (new_dict_type == nullptr)
-            Py_FatalError("PySide6: Problem creating ChameleonDict");
+            Py_FatalError("libshiboken: Problem creating ChameleonDict");
     }
 }
 
@@ -270,19 +274,20 @@ static inline void SelectFeatureSetSubtype(PyTypeObject *type, int select_id)
      * This is the selector for one sublass. We need to call this for
      * every subclass until no more subclasses or reaching the wanted id.
      */
-    static const auto *pyTypeType_tp_dict = PepType_GetDict(&PyType_Type);
+    static auto *pyTypeType_tp_dict = PepType_GetDict(&PyType_Type);
     AutoDecRef tpDict(PepType_GetDict(type));
     if (Py_TYPE(tpDict.object()) == Py_TYPE(pyTypeType_tp_dict)) {
         // On first touch, we initialize the dynamic naming.
         // The dict type will be replaced after the first call.
         if (!replaceClassDict(type)) {
-            Py_FatalError("failed to replace class dict!");
+            Py_FatalError("libshiboken: failed to replace class dict!");
             return;
         }
     }
     if (!moveToFeatureSet(type, select_id)) {
         if (!createNewFeatureSet(type, select_id)) {
-            Py_FatalError("failed to create a new feature set!");
+            PyErr_Print();
+            Py_FatalError("libshiboken: failed to create a new feature set!");
             return;
         }
     }
@@ -295,13 +300,12 @@ static inline int getFeatureSelectId()
 {
     static auto *undef = PyLong_FromLong(-1);
     static auto *feature_dict = GetFeatureDict();
-    // these things are all borrowed
-    auto *globals = PyEval_GetGlobals();
-    if (globals == nullptr
-        || globals == cached_globals)
+
+    Shiboken::AutoDecRef globals(PepEval_GetFrameGlobals());
+    if (globals.isNull() || globals.object() == cached_globals)
         return last_select_id;
 
-    auto *modname = PyDict_GetItem(globals, PyMagicName::name());
+    auto *modname = PyDict_GetItem(globals.object(), PyMagicName::name());
     if (modname == nullptr)
         return last_select_id;
 
@@ -326,12 +330,12 @@ static inline void SelectFeatureSet(PyTypeObject *type)
      * Generated functions call this directly.
      * Shiboken will assign it via a public hook of `basewrapper.cpp`.
      */
-    static const auto *pyTypeType_tp_dict = PepType_GetDict(&PyType_Type);
+    static auto *pyTypeType_tp_dict = PepType_GetDict(&PyType_Type);
     AutoDecRef tpDict(PepType_GetDict(type));
     if (Py_TYPE(tpDict.object()) == Py_TYPE(pyTypeType_tp_dict)) {
         // We initialize the dynamic features by using our own dict type.
         if (!replaceClassDict(type)) {
-            Py_FatalError("failed to replace class dict!");
+            Py_FatalError("libshiboken: failed to replace class dict!");
             return;
         }
     }
@@ -347,10 +351,10 @@ static inline void SelectFeatureSet(PyTypeObject *type)
     last_select_id = select_id;
 
     auto *mro = type->tp_mro;
-    const Py_ssize_t n = PyTuple_GET_SIZE(mro);
+    const Py_ssize_t n = PyTuple_Size(mro);
     // We leave 'Shiboken.Object' and 'object' alone, therefore "n - 2".
     for (Py_ssize_t idx = 0; idx < n - 2; idx++) {
-        auto *sub_type = reinterpret_cast<PyTypeObject *>(PyTuple_GET_ITEM(mro, idx));
+        auto *sub_type = reinterpret_cast<PyTypeObject *>(PyTuple_GetItem(mro, idx));
         SelectFeatureSetSubtype(sub_type, select_id);
     }
     // PYSIDE-1436: Clear all caches for the type and subtypes.
@@ -444,9 +448,9 @@ static PyObject *methodWithNewName(PyTypeObject *type,
      * Create a method with a lower case name.
      */
     auto *obtype = reinterpret_cast<PyObject *>(type);
-    const auto len = strlen(new_name);
+    const auto len = std::strlen(new_name);
     auto *name = new char[len + 1];
-    strcpy(name, new_name);
+    std::strcpy(name, new_name);
     auto *new_meth = new PyMethodDef;
     new_meth->ml_name = name;
     new_meth->ml_meth = meth->ml_meth;
@@ -627,10 +631,10 @@ static QByteArrayList GetPropertyStringsMro(PyTypeObject *type)
     auto res = QByteArrayList();
 
     PyObject *mro = type->tp_mro;
-    const Py_ssize_t n = PyTuple_GET_SIZE(mro);
+    const Py_ssize_t n = PyTuple_Size(mro);
     // We leave 'Shiboken.Object' and 'object' alone, therefore "n - 2".
     for (Py_ssize_t idx = 0; idx < n - 2; idx++) {
-        auto *subType = reinterpret_cast<PyTypeObject *>(PyTuple_GET_ITEM(mro, idx));
+        auto *subType = reinterpret_cast<PyTypeObject *>(PyTuple_GetItem(mro, idx));
         auto *props = SbkObjectType_GetPropertyStrings(subType);
         if (props != nullptr)
             for (; *props != nullptr; ++props)
