@@ -30,6 +30,8 @@
 #include <gilstate.h>
 #include <helper.h>
 #include <sbkconverter.h>
+#include <sbkerrors.h>
+#include <sbkpep.h>
 #include <sbkstring.h>
 #include <sbkstaticstrings.h>
 #include <sbkfeature_base.h>
@@ -63,7 +65,10 @@
 using namespace Qt::StringLiterals;
 
 static QStack<PySide::CleanupFunction> cleanupFunctionList;
-static void *qobjectNextAddr;
+
+// Used by QML (main thread), but needs to be protected against other
+// threads constructing QObject's.
+static void thread_local *qobjectNextAddr;
 
 QT_BEGIN_NAMESPACE
 extern bool qRegisterResourceData(int, const unsigned char *, const unsigned char *,
@@ -410,7 +415,8 @@ static void destructionVisitor(SbkObject *pyObj, void *data)
     auto *pyQApp = reinterpret_cast<SbkObject *>(realData[0]);
     auto *pyQObjectType = reinterpret_cast<PyTypeObject *>(realData[1]);
 
-    if (pyObj != pyQApp && PyObject_TypeCheck(pyObj, pyQObjectType)) {
+    auto *ob = reinterpret_cast<PyObject *>(pyObj);
+    if (pyObj != pyQApp && PyObject_TypeCheck(ob, pyQObjectType)) {
         if (Shiboken::Object::hasOwnership(pyObj) && Shiboken::Object::isValid(pyObj, false)) {
             Shiboken::Object::setValidCpp(pyObj, false);
 
@@ -595,10 +601,7 @@ PyObject *getHiddenDataFromQObject(QObject *cppSelf, PyObject *self, PyObject *n
 
     // Search on metaobject (avoid internal attributes started with '__')
     if (!attr) {
-        PyObject *type{};
-        PyObject *value{};
-        PyObject *traceback{};
-        PyErr_Fetch(&type, &value, &traceback);     // This was omitted for a loong time.
+        Shiboken::Errors::Stash errorStash;
 
         int flags = currentSelectId(Py_TYPE(self));
         int snake_flag = flags & 0x01;
@@ -623,8 +626,10 @@ PyObject *getHiddenDataFromQObject(QObject *cppSelf, PyObject *self, PyObject *n
                     if (res) {
                         AutoDecRef elemName(PyObject_GetAttr(res, PySideMagicName::name()));
                         // Note: This comparison works because of interned strings.
-                        if (elemName == name)
+                        if (elemName == name) {
+                            errorStash.release();
                             return res;
+                        }
                         Py_DECREF(res);
                     }
                     PyErr_Clear();
@@ -655,6 +660,7 @@ PyObject *getHiddenDataFromQObject(QObject *cppSelf, PyObject *self, PyObject *n
                     } else if (auto *func = MetaFunction::newObject(cppSelf, i)) {
                         auto *result = reinterpret_cast<PyObject *>(func);
                         PyObject_SetAttr(self, name, result);
+                        errorStash.release();
                         return result;
                     }
                 }
@@ -663,17 +669,17 @@ PyObject *getHiddenDataFromQObject(QObject *cppSelf, PyObject *self, PyObject *n
                 auto *pySignal = reinterpret_cast<PyObject *>(
                     Signal::newObjectFromMethod(cppSelf, self, signalList));
                 PyObject_SetAttr(self, name, pySignal);
+                errorStash.release();
                 return pySignal;
             }
         }
-        PyErr_Restore(type, value, traceback);
     }
     return attr;
 }
 
 bool inherits(PyTypeObject *objType, const char *class_name)
 {
-    if (strcmp(objType->tp_name, class_name) == 0)
+    if (std::strcmp(PepType_GetFullyQualifiedNameStr(objType), class_name) == 0)
         return true;
 
     PyTypeObject *base = objType->tp_base;
@@ -1058,10 +1064,10 @@ QMetaType qMetaTypeFromPyType(PyTypeObject *pyType)
         return QMetaType(QMetaType::Int);
     if (Shiboken::ObjectType::checkType(pyType))
         return QMetaType::fromName(Shiboken::ObjectType::getOriginalName(pyType));
-    return QMetaType::fromName(pyType->tp_name);
+    return QMetaType::fromName(PepType_GetFullyQualifiedNameStr(pyType));
 }
 
-debugPyTypeObject::debugPyTypeObject(const PyTypeObject *o) noexcept
+debugPyTypeObject::debugPyTypeObject(PyTypeObject *o) noexcept
     : m_object(o)
 {
 }
@@ -1073,7 +1079,7 @@ QDebug operator<<(QDebug debug, const debugPyTypeObject &o)
     debug.nospace();
     debug << "PyTypeObject(";
     if (o.m_object)
-        debug << '"' << o.m_object->tp_name << '"';
+        debug << '"' << PepType_GetFullyQualifiedNameStr(o.m_object) << '"';
     else
         debug << '0';
     debug << ')';
@@ -1210,10 +1216,6 @@ QDebug operator<<(QDebug debug, const debugPyObject &o)
     return debug;
 }
 
-debugPyBuffer::debugPyBuffer(Py_buffer *b) noexcept : m_buffer(b)
-{
-}
-
 static void formatPy_ssizeArray(QDebug &debug, const char *name, const Py_ssize_t *array, int len)
 {
     debug << ", " << name << '=';
@@ -1227,6 +1229,11 @@ static void formatPy_ssizeArray(QDebug &debug, const char *name, const Py_ssize_
     }
 }
 
+#if !defined(Py_LIMITED_API) || Py_LIMITED_API >= 0x030B0000
+debugPyBuffer::debugPyBuffer(Py_buffer *b) noexcept : m_buffer(b)
+{
+}
+
 PYSIDE_API QDebug operator<<(QDebug debug, const debugPyBuffer &b)
 {
     QDebugStateSaver saver(debug);
@@ -1252,5 +1259,6 @@ PYSIDE_API QDebug operator<<(QDebug debug, const debugPyBuffer &b)
     debug << ')';
     return debug;
 }
+#endif // !Py_LIMITED_API || >= 3.11
 
 } // namespace PySide
