@@ -14,7 +14,7 @@
 #include <overloaddata.h>
 #include <smartpointertypeentry.h>
 
-#include <QtCore/QDebug>
+#include <QtCore/qdebug.h>
 
 using namespace Qt::StringLiterals;
 
@@ -96,7 +96,7 @@ void CppGenerator::generateSmartPointerClass(TextStream &s,
                                              const GeneratorContext &classContext)
 {
     s.setLanguage(TextStream::Language::Cpp);
-    AbstractMetaClassCPtr metaClass = classContext.metaClass();
+    const AbstractMetaClassCPtr &metaClass = classContext.metaClass();
     const auto typeEntry = std::static_pointer_cast<const SmartPointerTypeEntry>(metaClass->typeEntry());
     const bool hasPointeeClass = classContext.pointeeClass() != nullptr;
     const auto smartPointerType = typeEntry->smartPointerType();
@@ -210,6 +210,7 @@ void CppGenerator::generateSmartPointerClass(TextStream &s,
     writeTpTraverseFunction(s, metaClass);
     writeTpClearFunction(s, metaClass);
 
+    writeClassTypeFunction(s, metaClass);
     writeClassDefinition(s, metaClass, classContext);
 
     s << '\n';
@@ -231,13 +232,16 @@ void CppGenerator::generateSmartPointerClass(TextStream &s,
 void CppGenerator::writeSmartPointerConverterFunctions(TextStream &s,
                                                        const AbstractMetaType &smartPointerType) const
 {
+    auto smartPointerTypeEntry =
+        std::static_pointer_cast<const SmartPointerTypeEntry>(smartPointerType.typeEntry());
+
+    if (smartPointerTypeEntry->hasCustomConversion())
+        writePythonToCppConversionFunctions(s, smartPointerType);
+
     const auto baseClasses = findSmartPointeeBaseClasses(api(), smartPointerType);
     if (baseClasses.isEmpty())
         return;
 
-    auto smartPointerTypeEntry =
-        std::static_pointer_cast<const SmartPointerTypeEntry>(smartPointerType.typeEntry());
-
     // TODO: Missing conversion to smart pointer pointer type:
 
     s << "// Register smartpointer conversion for all derived classes\n";
@@ -289,6 +293,8 @@ void CppGenerator::writeSmartPointerConverterInitialization(TextStream &s,
         writeAddPythonToCppConversion(s, targetConverter, toCpp, isConv);
     };
 
+    writeTemplateCustomConverterRegister(s, type);
+
     const auto classes = findSmartPointeeBaseClasses(api(), type);
     if (classes.isEmpty())
         return;
@@ -321,7 +327,7 @@ void CppGenerator::writeSmartPointerRichCompareFunction(TextStream &s,
     static const char selfPointeeVar[] = "cppSelfPointee";
     static const char cppArg0PointeeVar[] = "cppArg0Pointee";
 
-    const auto metaClass = context.metaClass();
+    const auto &metaClass = context.metaClass();
     QString baseName = cpythonBaseName(metaClass);
     writeRichCompareFunctionHeader(s, baseName, context);
 
@@ -410,7 +416,7 @@ void CppGenerator::writeSmartPointerGetattroFunction(TextStream &s,
                                                      const BoolCastFunctionOptional &boolCast)
 {
     Q_ASSERT(context.forSmartPointer());
-    const auto metaClass = context.metaClass();
+    const auto &metaClass = context.metaClass();
     writeGetattroDefinition(s, metaClass);
     s << "PyObject *tmp = PyObject_GenericGetAttr(self, name);\n"
       << "if (tmp)\n" << indent << "return tmp;\n" << outdent
