@@ -2,6 +2,7 @@
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "sbkenum.h"
+#include "sbkpep.h"
 #include "sbkstring.h"
 #include "helper.h"
 #include "sbkstaticstrings.h"
@@ -67,7 +68,7 @@ PyTypeObject *getPyEnumMeta()
             return reinterpret_cast<PyTypeObject *>(PyEnumMeta);
         }
     }
-    Py_FatalError("Python module 'enum' not found");
+    Py_FatalError("libshiboken: Python module 'enum' not found");
     return nullptr;
 }
 
@@ -77,7 +78,7 @@ void init_enum()
     if (isInitialized)
         return;
     if (!(isInitialized || _init_enum()))
-        Py_FatalError("could not init enum");
+        Py_FatalError("libshiboken: could not init enum");
 
     // PYSIDE-1735: Determine whether we should use the old or the new enum implementation.
     static PyObject *option = PySys_GetObject("pyside6_option_python_enum");
@@ -224,7 +225,7 @@ bool checkType(PyTypeObject *pyTypeObj)
     init_enum();
 
     static PyTypeObject *meta = getPyEnumMeta();
-    return Py_TYPE(pyTypeObj) == reinterpret_cast<PyTypeObject *>(meta);
+    return Py_TYPE(reinterpret_cast<PyObject *>(pyTypeObj)) == meta;
 }
 
 PyObject *getEnumItemFromValue(PyTypeObject *enumType, EnumValueType itemValue)
@@ -282,14 +283,14 @@ void setTypeConverter(PyTypeObject *type, SbkConverter *converter,
 
 static void setModuleAndQualnameOnType(PyObject *type, const char *fullName)
 {
-    const char *colon = strchr(fullName, ':');
+    const char *colon = std::strchr(fullName, ':');
     assert(colon);
     int package_level = atoi(fullName);
     const char *mod = colon + 1;
 
     const char *qual = mod;
     for (int idx = package_level; idx > 0; --idx) {
-        const char *dot = strchr(qual, '.');
+        const char *dot = std::strchr(qual, '.');
         if (!dot)
             break;
         qual = dot + 1;
@@ -306,7 +307,7 @@ static PyTypeObject *createEnumForPython(PyObject *scopeOrModule,
                                          const char *fullName,
                                          PyObject *pyEnumItems)
 {
-    const char *dot = strrchr(fullName, '.');
+    const char *dot = std::strrchr(fullName, '.');
     AutoDecRef name(Shiboken::String::fromCString(dot ? dot + 1 : fullName));
 
     static PyObject *enumName = String::createStaticString("IntEnum");
@@ -473,7 +474,7 @@ PyTypeObject *createPythonEnum(const char *fullName, PyObject *pyEnumItems,
         return nullptr;
     }
 
-    const char *dot = strrchr(fullName, '.');
+    const char *dot = std::strrchr(fullName, '.');
     AutoDecRef name(Shiboken::String::fromCString(dot ? dot + 1 : fullName));
     AutoDecRef callArgs(Py_BuildValue("(OO)", name.object(), pyEnumItems));
     auto *newType = PyObject_Call(PyEnumType, callArgs, callDict);
